% script for the demonstration of the different Fourier transforms FT,DTFT
% and DFT
%
% Cours Signaux et Syst?mes, Profs Vandergheynst & Unser
% Matthieu Guerquin-Kern, 2008-04-15, revised 2009-04-02

clear all;close all;clc;

%% Settings

dataname = 'montagne2.jpg';
R = 16; % undersampling factor

% List of figures:
data                    = 1;
OneDSignalinTime        = 2;
OneDSignalinFreq        = 3;
DiscretePeriodInTime    = 4;
DiscretePeriodInFreq    = 5;

% Color conventions
CTNP    = 'b'; % continuous time and non periodic signal = in blue
DTNP    = 'g'; % discrete time and non periodic signal   = in green
DTP     = 'r'; % discrete time and periodized signal     = in red
FCDP    = 'c'; % Fourier coefficients of the DTP         = in cyan
FFT     = 'k'; % DFT coefficient                         = in black

% Fourier representation
%fourierplot = @semilogy;
fourierplot = @plot;

fontsize = 16;

%% Getting the original "analogic" signal
% (in practice sampled fine enough to consider it as analogic)
Im = double(imread('montagne2.jpg'));
figure(data);
imagesc(Im);axis image;colormap gray;title('original data (HR image)','FontSize',fontsize);
hold on;plot([0, size(Im,2)-1],[round(size(Im,1)/2),round(size(Im,1)/2)],[CTNP '-'],'LineWidth',3);hold off;

% [Y,FS]=wavread(dataname);
% figure(data);
% plot(Y);

disp('High resolution data that represent fairly well the underlying continuous signal...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% extraction of a 1D "analogic" signal as a line of the image
x = Im(round(size(Im,1)/2),1:end);clear Im;
T = length(x)/16;      % signal length in arbitrary units (seconds, meters, pixels, etc)
dT = T/length(x);

% x = Y(10000:600000,1);
% dT = 1/FS;
% T = length(x)*dT;

disp('a 1D "continuous" signal is extracted from these data...');

%%  "Continuous" representations
% display the signal in space domain
t = (0:length(x)-1)*T/length(x);    % time axis
figure(OneDSignalinTime);
plot(t,x,[CTNP '-'],'LineWidth',2);title('1D "analogic" signal extracted from the data','FontSize',fontsize);
labelx = '"time"';
if T == 1
    labelx = ['normalized ' labelx];
end
xlabel(labelx,'FontSize',16);ylabel('Intensity','FontSize',fontsize);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Fourier transform
FT = fft(x);
F = 1/dT;
dF = F/length(x);
f = (-length(x)/2:length(x)/2-1)*F/length(x);   % frequency axis

% display the signal in the frequency domain
figure(OneDSignalinFreq);
fourierplot(f,fftshift(abs(FT)),[CTNP '-'],'LineWidth',2);title('Fourier transform for the 1D signal','FontSize',fontsize);
labelx = 'frequency';
if F == 1
    labelx = ['normalized ' labelx];
end
xlabel(labelx,'FontSize',fontsize);
ylabel('modulus','FontSize',fontsize);

disp('Representation of the "continuous" Fourier transform...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Sampling (in practice just an undersampling)

x_echR = x(1:R:length(x));
x_ech = zeros(1,length(x));
x_ech(1:R:length(x)) = x_echR;
tR = t(1:R:length(t));

% display the sampled signal in space domain
figure(OneDSignalinTime);
hold on;stem(tR,x_echR,[DTNP]);hold off;legend('"continous" signal','sampled signal');
title('continous representation');

disp('The "continuous" signal is sampled. In practice we apply a 16 times undersampling...');
disp('This sampled signal is still represented in "continuous" time by diracs...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% FT of the sampled signal (still in continous time)
DTFT = R*fft(x_ech);

% display the sampled signal in frequency domain
figure(OneDSignalinFreq);
hold on;fourierplot(f,fftshift(abs(DTFT)),[DTNP '-']);legend('Fourier transform of the continous signal','DTFT of the sampled signal');hold off;

disp('The Fourier transform of the sampled signal is a periodized version of the Fourier transform of the original continuous signal...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Periodization of the sampled signal
x_echR_per = repmat(x_echR,1,R);

% display the periodized sampled signal in space domain
figure(DiscretePeriodInTime);
hold on;stem(1:length(x_echR),x_echR,[DTNP],'LineWidth',2);ylabel('Intensity','FontSize',fontsize);xlabel('discrete "time"','FontSize',fontsize);hold off;
title('discrete representation');

disp('Discrete time representation of the sampled signal...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(DiscretePeriodInTime);
hold on;
stem(1:length(x_echR_per),x_echR_per,[DTP],'LineWidth',1);legend('sampled signal','sampled signal periodized');
hold off;

disp('Periodization of the sampled signal...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DTFT_per = fft(x_echR_per);

% display the periodized sampled signal in frequency domain
figure(DiscretePeriodInFreq);
index = find((f>-0.5)&(f<=0.5));
tmp = fftshift(abs(DTFT));
plot(f(index),tmp(index),[DTNP '-'],'LineWidth',2);legend('pattern of the Fourier transform of the sampled signal');
xlabel('normalized frequency (continuous)','FontSize',fontsize);
ylabel('modulus','FontSize',fontsize);

disp('Representation of the pattern of the periodic Fourier transform of the sampled (continuous) signal...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hold on;
plot(f/F,fftshift(abs(DTFT_per)),[DTP '-'],'LineWidth',2);
legend('pattern of the Fourier transform of the sampled signal','DTFT of the periodized sampled signal');
hold off;

disp('Discrete time Fourier transform of the periodized discrete signal -> sampling of the pattern in Fourier...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% DFT (the Fourier coefficients of the periodized sampled signal)
DFT = DTFT_per(1:R:end);
fR = f(1:R:end);

% display the Fourier coefficients in frequency axis
figure(DiscretePeriodInFreq);
hold on;stem(fR/F,fftshift(abs(DFT)),[FCDP],'LineWidth',2);
legend('pattern of the Fourier transform of the sampled signal','DTFT of the periodized sampled signal','Fourier coefficients (DFT)');hold off;

disp('Fourier coefficients of the periodized discrete signal -> samples of the pattern in Fourier...');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
pause %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Comparison with the FFT
relative_error = mean(abs(DFT-R*fft(x_echR)).^2)/mean(abs(DFT).^2)
figure(DiscretePeriodInFreq);
hold on;stem(fR/F,R*fftshift(abs(fft(x_echR))),[FFT],'LineWidth',1);legend('pattern of the Fourier transform of the sampled signal','DTFT of the periodized sampled signal','Fourier coefficients (DFT)','FFT of the sampled signal');hold off;
disp('The DFT coefficients correspond to the latter...');